precision highp float;
uniform mat4 uMVPMatrix;
varying float xPostion;
varying vec2  v_texCoord;
uniform sampler2D u_texture;
uniform  float u_timeScaled;


float random (in vec2 _st) {
    float a = 12.9898;
    float b = 78.233;
    float c = 43758.5453;
    float dt= dot(_st.xy, vec2(a, b));
    float sn= mod(dt, 3.14);
    return fract(sin(sn) * c);
}

// Based on Morgan McGuire @morgan3d
// https://www.shadertoy.com/view/4dS3Wd
float noise (in vec2 _st) {
    vec2 i = floor(_st);
    vec2 f = fract(_st);

    // Four corners in 2D of a tile
    float a = random(i);
    float b = random(i + vec2(1.0, 0.0));
    float c = random(i + vec2(0.0, 1.0));
    float d = random(i + vec2(1.0, 1.0));

    vec2 u = f * f * (3.0 - 2.0 * f);

    return mix(a, b, u.x) + (c - a)* u.y * (1.0 - u.x) + (d - b) * u.x * u.y;
}


float fbm (in vec2 _st) {
    float v = 0.0;
    float a = 0.5;
    vec2 shift = vec2(1.0);
    shift = _st * 0.2;
    // Rotate to reduce axial bias
    mat2 rot = mat2(cos(0.5), sin(0.5), -sin(0.5), cos(0.5));
    for (int i = 0; i < 3; ++i) {
        v += a * noise(_st);
        _st = rot * _st * 1.4 + shift;
        a *= 0.7;
    }
    return v;
}
vec3 rgb2hsv(vec3 c)
{
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}
vec3 hsv2rgb(vec3 c)
{
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

vec3 hsl2rgb(in vec3 c)
{
    vec3 rgb = clamp(abs(mod(c.x*6.0+vec3(0.0, 4.0, 2.0), 6.0)-3.0)-1.0, 0.0, 1.0);

    return c.z + c.y * (rgb-0.5)*(1.0-abs(2.0*c.z-1.0));
}

vec3 rgb2hsl(in vec3 c){
    float h = 0.0;
    float s = 0.0;
    float l = 0.0;
    float r = c.r;
    float g = c.g;
    float b = c.b;
    float cMin = min(r, min(g, b));
    float cMax = max(r, max(g, b));

    l = (cMax + cMin) / 2.0;
    if (cMax > cMin) {
        float cDelta = cMax - cMin;

        //s = l < .05 ? cDelta / ( cMax + cMin ) : cDelta / ( 2.0 - ( cMax + cMin ) ); Original
        s = l < .0 ? cDelta / (cMax + cMin) : cDelta / (2.0 - (cMax + cMin));

        if (r == cMax) {
            h = (g - b) / cDelta;
        } else if (g == cMax) {
            h = 2.0 + (b - r) / cDelta;
        } else {
            h = 4.0 + (r - g) / cDelta;
        }

        if (h < 0.0) {
            h += 6.0;
        }
        h = h / 6.0;
    }
    return vec3(h, s, l);
}

void main() {
    vec2 uv = v_texCoord;


    vec2 q = vec2(fbm(uv + vec2(u_timeScaled*0.1, cos(u_timeScaled*0.2))), fbm(uv + vec2(u_timeScaled*0.1, u_timeScaled)));

    vec2 r = vec2(
    fbm(0.5*uv + 1.5*q + vec2(u_timeScaled*0.05, 0.0)),
    fbm(0.5*uv + 3.0*q + vec2(u_timeScaled*0.1, 0.0)));

    float u_scale = 1.0;
    float u_border = 0.1;

    r = (r-vec2(0.5, 0.5)) * (u_scale + u_border) + vec2(0.5, 0.5);// make uv to corner

    vec4 texture_color = vec4(84.0 / 255.0, 123.0/255.0, 100.0/255.0, 1.0);
    vec3 hsv = rgb2hsl(texture_color.xyz);
    float l1 =  r.y * 0.6 + 0.2;
    /*
        float l2 =  max(0.2, l1);
        l2 = min(0.8, l2);
    */
    vec3 rgb = hsl2rgb(vec3(hsv.x, hsv.y, l1));

    gl_FragColor = vec4(rgb, 1.0);
}
